/*
* Copyright 2004-2005 mbsx@seznam.cz
* All Rights Reserved.
*/
using System;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Data;
using System.Collections;
using System.Drawing;
using System.Resources;
using System.IO;
using System.Security;
using System.Reflection;
using System.Xml;
using System.Threading;
using System.Diagnostics;
using System.Drawing.Printing;
using Outlook = Microsoft.Office.Interop.Outlook;


namespace DeskNotes
{
	
	//public delegate void Saver();

	/// <summary>
	/// DeskNotes provides an access to underlying DataSet DeskNote and to XML file which
	/// stores actual data.
	/// It performs reading and initiating of notes collection on startup.
	/// Adding/updating/removing of notes during application usage through simplified interface.
	/// </summary>
	public class DeskNotes : ArrayList
	{
        public enum UsageType { FirstTime, OldVersionFound, Regular };

		public static string DEBUG_REGISTRY ="Software\\M.B.\\DeskNotes\\Debug\\Main";
		const string IMEASURE_KEY = "Control Panel\\International";
		const string IMEASURE_VALUE = "iMeasure";
		const double INCH_MM = 2.54;
        internal const string DEFAULT_DATA = "DeskNotes.xml";
        internal const string DEFAULT_ARCHIVE = "DeskNotes_Archive.xml";

		DeskNote dsDeskNotes = null;
		private ArchivedNotes archive = null;
		private RemindersForm remindersForm = null;
		
		private Color oldBackColor;
		private Color oldTextColor;
        private Font oldFont;
        private bool oldDataFiles = false;
		
		private bool showEmptyNote = false;
        private System.Drawing.Printing.PageSettings pageSettings = new System.Drawing.Printing.PageSettings();

       
		internal ArchiveForm archiveForm = null;
		private static bool debug = false;
		private System.Windows.Forms.ToolStripMenuItem[] disabledPrintMenus;
		private NotePrintDocument notePrintDoc;
		private bool showingAllNotes = true;
        
        private bool outlookOK = false;

        internal bool OutlookOK
        {
            get { return outlookOK; }
            set { outlookOK = value; }
        }
        
		
		public static string Debug
		{
			set 
			{
				if (("debug" == value) || ("Debug" == value) || ("DEBUG" == value))
					debug = true;
			}
		}
        
		internal bool ShowingAllNotes
		{
			get 
			{
				return showingAllNotes;
			}
			set 
			{
				showingAllNotes = value;
			}
		}

	
		public ArchivedNotes Archive
		{
			get 
			{
				return archive;
			}
		}

        public bool PlaySound 
		{
			get 
			{
				return "" != Properties.Settings.Default.SoundFile ? true : false;
			}
		}

		
		/// <summary>
		/// Property tells the application if it is first start on users desktop.
        /// or older version DeskNotes data files are available or it is regular consequent start.
		/// </summary>
		public UsageType UsageStyle
		{
			get
			{
                if (oldDataFiles)
                    return UsageType.OldVersionFound;
				else if (Properties.Settings.Default.DataFile == "") 
					return UsageType.FirstTime;
				else
					return UsageType.Regular;
			}
		}

        internal System.Drawing.Printing.PageSettings PageSettings
        {
            set
            {
                pageSettings = value;
                Properties.Settings.Default.PrintMargins = value.Margins;
            }
            get
            {
                return pageSettings;
            }
        }

        /// <summary>
        /// Use when print margins are displayed to user. They are calculated to user measurement system.
        /// </summary>
		internal System.Drawing.Printing.PageSettings DisplayedPageSettings 
		{
			get
			{
				if (IsInchesMeasurement()){
					return pageSettings;
				}
				else { //metrics system
                    
                    PageSettings pS = new PageSettings();
                    pS = (PageSettings) pageSettings.Clone();
                    Margins m = pS.Margins;

					pS.Margins.Top = (int)(m.Top * INCH_MM);
					pS.Margins.Left = (int)(m.Left * INCH_MM);
					pS.Margins.Right = (int)(m.Right * INCH_MM);
					pS.Margins.Bottom = (int)(m.Bottom * INCH_MM);
					return pS;
                }
			}
		}

		/// <summary>
		/// Get info about measurement system of the current user.
		/// </summary>
		/// <returns>true if measurement is inches (US), false if metrics</returns>
		internal bool IsInchesMeasurement()
		{
			short measureInches = 1;
			RegistryKey key = Registry.CurrentUser;
			try
			{
				key = key.OpenSubKey(IMEASURE_KEY);
				measureInches = Int16.Parse((string)key.GetValue(IMEASURE_VALUE));
			} 
			catch (Exception e)
			{
				System.Console.WriteLine("Exception reading iMeasure info: " + e.Message);
			}
			finally 
			{
				if (null != key)
					key.Close();
			}
			if (1 == measureInches)
				return true;
			else
				return false;
		}

		/// <summary>
		/// Reads settings written by 1.0 version of DeskNotes
		/// </summary>
		/// <returns>true - settings were read, otherwise false</returns>
		internal bool Read1_0Settings()
		{
			RegistryKey mainKey = Registry.CurrentUser;
			try 
			{
				mainKey = mainKey.OpenSubKey(MainResource.GetString("REGISTRY_KEY_1_0"));
				// if key does not exist then application is used for first time.
				if (mainKey == null)
				{
					return false;
				}
				else
				{
                    Properties.Settings settings = Properties.Settings.Default;
					string cult = (string) mainKey.GetValue("Culture");
					if (null != cult)
					{
						settings.Culture = cult;
					}
					settings.DataFile = (string) mainKey.GetValue("DataFile");
					
					object color =  mainKey.GetValue("BackgroundColor");
					if (null != color)
						settings.BackgroundColor = Color.FromArgb(int.Parse(color.ToString()));
					color = mainKey.GetValue("TextColor");
					if (null != color)
						settings.TextColor = Color.FromArgb(int.Parse(color.ToString()));
					string sound = mainKey.GetValue("SoundFile").ToString();
					if ((null != sound ) && ("" != sound))
						settings.SoundFile = sound;
					string op = (string)mainKey.GetValue("Opacity");
					if (null != color)
						settings.Opacity = double.Parse(op.ToString());

					string fontName = (string)mainKey.GetValue("Font.Name");
					if (null != fontName)
					{
						float size = Single.Parse((string)mainKey.GetValue("Font.Size",10));

						FontStyle fontStyle = FontStyle.Regular;
						bool bold = Boolean.Parse((string)mainKey.GetValue("Font.Bold","false"));
						bool italic = Boolean.Parse((string)mainKey.GetValue("Font.Italic","false"));
						bool strikeout = Boolean.Parse((string)mainKey.GetValue("Font.Strikeout","false"));
						bool underline = Boolean.Parse((string)mainKey.GetValue("Font.Underline","false"));
						fontStyle |= bold ? FontStyle.Bold : 0;
						fontStyle |= italic ? FontStyle.Italic : 0;
						fontStyle |= strikeout ? FontStyle.Strikeout : 0;
						fontStyle |= underline ? FontStyle.Underline : 0;
						settings.Font = new Font(fontName, size,fontStyle);
					}
					return true;
				}
			}
			catch (Exception e)
			{
				MessageBox.Show(MainResource.GetString("ERROR_READ_SETTINGS") + e.Message, 
					MainResource.GetString("APP_TITLE"),
					MessageBoxButtons.OK,MessageBoxIcon.Error);
				return false;
			} 
			finally 
			{
				if (null != mainKey)
					mainKey.Close();
			}
		}

        /// <summary>
        /// Reads settings from registry for version 1.5.
        /// </summary>
        /// <returns></returns>
        internal bool Read_1_5_Settings()
        {
            RegistryKey mainKey = Registry.CurrentUser;
            try
            {
                if (DeskNotes.debug)
                    mainKey = mainKey.OpenSubKey(DeskNotes.DEBUG_REGISTRY);
                else
                    mainKey = mainKey.OpenSubKey(MainResource.GetString("REGISTRY_KEY"));

                // if key does not exist then application is used for first time.
                if (mainKey == null)
                {
                    return Read1_0Settings();
                }
                else
                {
                    Properties.Settings settings = Properties.Settings.Default;
                    string cult = (string)mainKey.GetValue("Culture");
                    if (null != cult)
                    {
                        settings.Culture = cult;
                    }
                    settings.DataFile = (string)mainKey.GetValue("DataFile");
                    //archive can be null if this Reg value is not defined.
                    settings.ArchiveFile = (string)mainKey.GetValue("ArchiveFile");

                    object color = mainKey.GetValue("BackgroundColor");
                    if (null != color)
                        settings.BackgroundColor = Color.FromArgb(int.Parse(color.ToString()));
                    color = mainKey.GetValue("TextColor");
                    if (null != color)
                        settings.TextColor = Color.FromArgb(int.Parse(color.ToString()));
                    string sound = (string) mainKey.GetValue("SoundFile");
                    if ((null != sound) && ("" != sound))
                        settings.SoundFile = sound;
                    string op = (string)mainKey.GetValue("Opacity");
                    if (null != op)
                        settings.Opacity = double.Parse(op.ToString());

                    object id = mainKey.GetValue("SelectedNodeId");
                    if (null != id)
                        settings.SelectedNodeId = long.Parse(id.ToString());
                    object type = mainKey.GetValue("SelectedNodeType");
                    if (null != type)
                        settings.SelectedNodeType = byte.Parse(type.ToString());

                    settings.InsertOnlyDate = Boolean.Parse((string)mainKey.GetValue("InsertOnlyDate", "false"));
                    settings.OriginalLook = Boolean.Parse((string)mainKey.GetValue("OriginalLook", "false"));
                    settings.DragDropToArchive = Boolean.Parse((string)mainKey.GetValue("DragDropToArchive", "false"));

                    string fontName = (string)mainKey.GetValue("Font.Name");
                    if (null != fontName)
                    {
                        float size = Single.Parse((string)mainKey.GetValue("Font.Size", 10));

                        FontStyle fontStyle = FontStyle.Regular;
                        bool bold = Boolean.Parse((string)mainKey.GetValue("Font.Bold", "false"));
                        bool italic = Boolean.Parse((string)mainKey.GetValue("Font.Italic", "false"));
                        bool strikeout = Boolean.Parse((string)mainKey.GetValue("Font.Strikeout", "false"));
                        bool underline = Boolean.Parse((string)mainKey.GetValue("Font.Underline", "false"));
                        fontStyle |= bold ? FontStyle.Bold : 0;
                        fontStyle |= italic ? FontStyle.Italic : 0;
                        fontStyle |= strikeout ? FontStyle.Strikeout : 0;
                        fontStyle |= underline ? FontStyle.Underline : 0;
                        settings.Font = new Font(fontName, size, fontStyle);
                    }
                    return true;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(MainResource.GetString("ERROR_READ_SETTINGS") + e.Message,
                    MainResource.GetString("APP_TITLE"),
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            finally
            {
                if (null != mainKey)
                    mainKey.Close();
            }
        }
	/// <summary>
	/// Constructor performs reading and initialization of notes from
	/// XML file on disk.
	/// </summary>
		public DeskNotes()
		{
            Properties.Settings settings = Properties.Settings.Default;
            try
            {
                if ("" == settings.DataFile)
                {   // 1st try to upgrade from previous version of 2.x.x.x DeskNotes
                    settings.Upgrade();
                }
                if ("" != settings.DataFile){
                    //read current settings if datafile is defined in config file
                    //archive can be null if this settings value is not defined.
                    //archiveFile = settings.ArchiveFile;
                    oldBackColor = settings.BackgroundColor;
                    oldTextColor = settings.TextColor;
                    oldFont = settings.Font;
                    if (null != settings.PrintMargins)
                        pageSettings.Margins = settings.PrintMargins;
                   
                    //Verify if Outlook is still installed & usable
                    try
                    {
                        Outlook.Application olApp = new Outlook.Application();
                        Outlook.NoteItem note = (Outlook.NoteItem)olApp.CreateItem(Microsoft.Office.Interop.Outlook.OlItemType.olNoteItem);
                        note.Delete();
                        note = null;
                        outlookOK = true;
                    }
                    catch (Exception e)
                    {
                        outlookOK = false;
                    }
                }
                else
                {   //read settings from previous version if datafile is not defined.
                    this.oldDataFiles = Read_1_5_Settings();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(MainResource.GetString("ERROR_READ_SETTINGS") + e.Message,
                    MainResource.GetString("APP_TITLE"),
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                //react to certain properties change after initial setup.
                settings.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(settings_PropertyChanged);
            }
		}

        /// <summary>
        /// When some of the properties has changed event is raised.
        /// Only few properties needs to be specially handled.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void settings_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            switch( e.PropertyName ){
                case null: //all properties has changed
                    break;
                case "BackgroundColor":
                    SetNotesBackgroundColor();
                    oldBackColor = Properties.Settings.Default.BackgroundColor;
                    break;
                case "TextColor":
                    SetNotesTextColor();
                    oldTextColor = Properties.Settings.Default.TextColor;
                    break;
                case "Opacity":
                    SetOpacity();
                    break;
                case "DataFile":
                    // XML file is checked by reading into temporary dataset only if file exists
                    if (File.Exists(Properties.Settings.Default.DataFile)){
                        DeskNote data = new DeskNote();
                        data.ReadXml(Properties.Settings.Default.DataFile, XmlReadMode.Auto);
                    }
                   break;
                case "Font":
                    SetNotesFont();
                    oldFont = Properties.Settings.Default.Font;
                    break;
                default:
                    break;
            }
            
        }

  		/// <summary>
		/// Method opens dataset for specified XML file. It handles all the errors which can happen 
		/// when reading the file on a disk.
		/// Method can throw an exception while reading the file.
		/// </summary>
		/// <returns>true if file was really open, otherwise false.</returns>
		internal bool OpenDataSet()
		{
			//perform data file checks
            if (File.Exists(Properties.Settings.Default.DataFile))
            {
                dsDeskNotes = new DeskNote();
                dsDeskNotes.ReadXml(Properties.Settings.Default.DataFile);
                return true;
            }
            else //dataset filename was specified but file does not exist
            {
                string text = string.Format(MainResource.GetString("DATA_FILE_DOES_NOT_EXIST"), Properties.Settings.Default.DataFile);
                if (DialogResult.Yes == MessageBox.Show( text, MainResource.GetString("APP_TITLE"),
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question))
                {
                    dsDeskNotes = new DeskNote();
                    dsDeskNotes.WriteXml(Properties.Settings.Default.DataFile, XmlWriteMode.WriteSchema);
                    showEmptyNote = true;
                    return true;
                }
            }
            return false;
		}

		/// <summary>
		/// Method creates new DataSet in specified filename
		/// </summary>
		/// <param name="filename">file to be used for DeskNotes file creation</param>
		/// <returns>true if file name was created, otherwise false</returns>
		public bool CreateNewDataSet(string fileName)
		{
            // Creates dataset object instance
            dsDeskNotes = new DeskNote();
            if (File.Exists(fileName))
            {
                try
                {
                    dsDeskNotes.ReadXml(fileName, XmlReadMode.ReadSchema);
                }
                catch (XmlException ex)
                {
                    Console.WriteLine("Read DaskNotes file error: " + ex.Message);
                    //Ask before overwriting existing non desknotes file
                    string text = string.Format(Resources.AppResource.FileExists, fileName);
                    if (DialogResult.Yes == MessageBox.Show(text, Resources.AppResource.APP_TITLE,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question))
                    {
                        dsDeskNotes.WriteXml(fileName, XmlWriteMode.WriteSchema);
                    }
                    else
                        return false; //Return file not created if users says No to Overwrite
                }
            }
            else // Archive file does not exist.
            {
                //create new archive file with ArchivedNotes schema assigned.
                //ArchivedNotes tempArch = new ArchivedNotes();
                dsDeskNotes.WriteXml(fileName, XmlWriteMode.WriteSchema);
            }
            showEmptyNote = true;
            return true;
            /*
			dsDeskNotes = new DeskNote();
			dsDeskNotes.WriteXml(Properties.Settings.Default.DataFile, XmlWriteMode.WriteSchema);
			//Properties.Settings.Default.DataFile = filename;
			showEmptyNote = true;
            */
		}

		/// <summary>
		/// Method reads all notes from data file and stores them inside this object.
		/// 
		/// </summary>
		public void ReadAllData()
		{
			DeskNote.DeskNoteDataTable tblDeskNotes = dsDeskNotes._DeskNote;
			if (tblDeskNotes.Count == 0)
				return;
            // List of IDs to be removed from corrupted Desknotes dataset
            long[] idToRemove = new long[tblDeskNotes.Count];
            int counter = 0;
			foreach (DeskNote.DeskNoteRow noteRow in tblDeskNotes.Rows)
			{
                Note note = new Note(noteRow.ID);
                try
                {
                    note.ReadFromDataSet(noteRow);
                    note.DataSet = dsDeskNotes;
                    this.Add(note);
                }
                catch (System.Data.StrongTypingException ste)
                {
                    // Remember corrupted record. It cannot be removed while it is used inside foreach() cycle.
                    if (noteRow.IstextNull())
                        idToRemove[counter++] = noteRow.ID;

                    note = null;
                }
			}
            // Remove corrupted desknotes data set recores
            if (counter > 0)
            {
                for (int i = 0; i < counter; i++)
                {
                    RemoveDeskNote(idToRemove[i]);
                }
            }
	    }

        /// <summary>
        /// Remove DeskNote by ID from dataset recordset.
        /// It is not removed from DeskNotes main class (a list)
        /// </summary>
        /// <param name="id">ID of desknote to be removed. Used to search for it.</param>
        /// <returns>True if removal succeeded</returns>
        internal bool RemoveDeskNote(long id)
        {
            DeskNote.DeskNoteDataTable tblDeskNotes = dsDeskNotes._DeskNote;
            if (tblDeskNotes.Count == 0)
                //throw exception or something.
                return false;

            // Use the Select method to find all rows matching the filter.
            DeskNote.DeskNoteRow[] foundRows = (DeskNote.DeskNoteRow[])tblDeskNotes.Select("ID = " + id.ToString());
            if (foundRows.Length >= 1)
            {
                foundRows[0].Delete();

                tblDeskNotes.AcceptChanges();
                //save all data when change is done.
                Save();
                return true;
            }
            return false;
        }

		/// <summary>
		/// Method performs showing of desknotes on a desktop.
		/// When there are no desk notes in the dataset then create a new one.
		/// </summary>
        internal void ShowDeskNotes()
		{
			if (0 != this.Count)
			{
				Note topMostNote = null;
				IEnumerator myEnum = this.GetEnumerator();
				while (myEnum.MoveNext())
				{	// Show all desknotes with exception of top most one. 
					// Only one desknote can be top most
					Note note = (Note)myEnum.Current;
					note.DataHandler = this;
					if (!note.OnTop)
					{
						NoteForm nf = new NoteForm();
						nf.NoteData = note;
						nf.Show();

						if (note.Hidden)
							nf.Hide();
					} 
					else if (note.OnTop)
					{  // remember top most desknote
						topMostNote = note;
					}
				}
				// show top most note as last one.
				if (null != topMostNote)
				{
					NoteForm nf = new NoteForm();
					nf.NoteData = topMostNote;
					nf.Show();
				}
			} 
			else if ((showEmptyNote) && (Properties.Settings.Default.ShowInitialDeskNote))
			{ // create an empty first desk note which contains just question to initiate notes adding.
				CreateNewNote(MainResource.GetString("FIRST_NOTE"), Note.DefaultPosition);
			}
			ShowingAllNotes = false;
		}
		/// <summary>
		/// Method creates new note when user choose it from running application
		/// </summary>
		public Note CreateNewNote(string text, Rectangle position)
		{
            Properties.Settings settings = Properties.Settings.Default;

			NoteForm nf = new NoteForm();
			Note note = new Note(0, text, position, settings.BackgroundColor, settings.TextColor);
			nf.NoteData = note;
			note.DataHandler = this;
			//note.Exiter = new ExitHandler(this.Exit);
			note.DataSet = dsDeskNotes;
			this.Add(note);
			note.Save();
			nf.Show();
			// Top most note is reset when note is added. It needs to
			// be set again after creation.
			ConfirmTopMost();
			//nf.Activate();
			return note;
		}

		/// <summary>
		/// Show Note on desktop from archived note.
		/// </summary>
		/// <param name="note">Note to be showed with new NoteForm created.</param>
		internal bool ShowArchivedNote(Note note)
		{
			NoteForm nf = new NoteForm();
			nf.NoteData = note;
			note.DataHandler = this;
			//note.Exiter = new ExitHandler(this.Exit);
			note.DataSet = dsDeskNotes;
			this.Add(note);
			note.Save();
			nf.Show();
			// Top most note is reset when note is added. It needs to
			// be set again after creation.
			ConfirmTopMost();
			return true;
		}

		/// <summary>
		/// Methods sets curren On top desk note again as top most. TopMost notes are reset
		/// when new DeskNote is added or hidden is made visible.
		/// </summary>
		internal void ConfirmTopMost()
		{
			IEnumerator myEnum = this.GetEnumerator();
			while (myEnum.MoveNext())
			{	
				Note topNote = (Note)myEnum.Current;
				if (topNote.OnTop)
				{
					topNote.MyForm.TopMost = true;
					break;
				}
			}
		}

		
		
		/// <summary>
		/// Method removes currently selected topmost desknote flag.
		/// This is necessary as only one form can be topmost and if 
		/// it is set to two then no one is top most.
		/// </summary>
		internal void ResetCurrentTopMost()
		{
			IEnumerator myEnum = this.GetEnumerator();
			while (myEnum.MoveNext())
			{	
				Note note = (Note)myEnum.Current;
				if (note.OnTop)
				{
					note.OnTop = false;
					note.MyForm.TopMost = false;
				}
			}
		}

		
		/// <summary>
		/// Save main desknotes data file.
		/// </summary>
		internal void Save()
		{
			try
			{
                if (null != dsDeskNotes)
                // Write to the file with the WriteXml method.
				dsDeskNotes.WriteXml(Properties.Settings.Default.DataFile, XmlWriteMode.WriteSchema);
			} 
			catch (SecurityException e)
			{
				string text = string.Format(MainResource.GetString("ERROR_WRITE_DATA"), Properties.Settings.Default.DataFile, e.Message + "\n");
				if ( DialogResult.No == MessageBox.Show(text, MainResource.GetString("APP_TITLE"),
						MessageBoxButtons.YesNo, MessageBoxIcon.Error))
					return;
			}
		}

		/// <summary>
		/// Method saves archive to disk if this was ever loaded during application lifetime.
		/// </summary>
		internal void SaveArchive()
		{
			if (null != archive)
			{
				// Write to the file with the WriteXml method.
				archive.WriteXml(Properties.Settings.Default.ArchiveFile, XmlWriteMode.WriteSchema);
			}
		}

		/// <summary>
		/// Causes exit event to be sent and all clean up & data saving to happen.
		/// </summary>
		internal void Exit()
		{
			Application.Exit();
		}

		/// <summary>
		/// Saves archive and application settings upon exit event.
		/// </summary>
		internal void SaveAll()
		{
			try {
                //Archive is saved only when DeskNotes exits.
                SaveArchive();
                //run through desknotes and save them all.
                IEnumerator myEnum = this.GetEnumerator();
                while (myEnum.MoveNext())
                {
                    Note note = (Note)myEnum.Current;
                    note.Save();
                }
                Save();
			} 
			catch (SystemException e)
			{
				Console.WriteLine("SaveArchive error: " + e.Message); 
				Console.WriteLine("   " + e.TargetSite.Name + "; " + e.StackTrace);
			}
			try {
                Properties.Settings.Default.Save();
			} 
			catch (Exception e)
			{
                MessageBox.Show(Resources.AppResource.ERROR_WRITE_SETTINGS + e.Message, Resources.AppResource.APP_TITLE, 
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

        
        /// <summary>
        /// Creates disk file for archive specified by user.
        /// </summary>
        /// <returns>true if file was created on a disk, otherwise false.IOException will be thrown for some disk errors</returns>
        internal bool CreateArchiveFile(string archiveFileName)
        {
            ArchivedNotes tempArch = new ArchivedNotes();
            if (File.Exists(archiveFileName))
            {
                try
                {
                    tempArch.ReadXml(archiveFileName, XmlReadMode.ReadSchema);
                }
                catch (XmlException ex)
                {
                    Console.WriteLine("Read Archive error: " + ex.Message);
                    //Ask before overwriting existing non archive file
                    string text = string.Format(Resources.AppResource.FileExists, archiveFileName);
                    if (DialogResult.Yes == MessageBox.Show(text, Resources.AppResource.APP_TITLE,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question))
                    {
                        tempArch.WriteXml(archiveFileName, XmlWriteMode.WriteSchema);
                        return true;
                    }
                    else
                        return false; //Return file not created if users says No to Overwrite
                }
            }
            else // Archive file does not exist.
            {
                //create new archive file with ArchivedNotes schema assigned.
                //ArchivedNotes tempArch = new ArchivedNotes();
                tempArch.WriteXml(archiveFileName, XmlWriteMode.WriteSchema);
                return true;
            }
            return true;
        }

		/// <summary>
		/// Method loads archive into memory or asks for its disk location for a very first time.
		/// When archive is loaded it's associated with ArchiveForm. ArchiveForm can be created only 
		/// once at DeskNotes. Once archive is loaded it is used forever with ArchiveForm
		/// </summary>
		/// <returns>true when archive was loaded into memory.</returns>
		internal bool LoadArchive()
		{
			try 
			{
				if (null == archive)
				{
					//return archiveForm;
                    string archiveFile = Properties.Settings.Default.ArchiveFile;
					if (!File.Exists(archiveFile))
					{
						OpenFileDialog of = new OpenFileDialog();
						of.Title = MainResource.GetString("ARCHIVE_FILE_OPEN");
                        of.FileName = archiveFile;
						of.CheckFileExists = false;
						of.DefaultExt = "xml";
						of.Filter = "XML file|*.xml|All files|*.*";
						if (DialogResult.OK == of.ShowDialog())
						{
							archiveFile = of.FileName;
                            if (CreateArchiveFile(archiveFile))
                            {
                                Properties.Settings.Default.ArchiveFile = archiveFile;
                                //create new archive file with ArchivedNotes schema assigned.
                                archive = new ArchivedNotes();
                                archive.ReadXml(archiveFile);
                                return true;
                            }
                            else
                            {
                                return false;
                            }
						} 
						else 
						{ // user escaped open file dialog without file selected. 
							// Show info to him and cancel whole action
							MessageBox.Show( MainResource.GetString("ERROR_ARCHIVE_FILE_NOT_SELECTED"),
								MainResource.GetString("APP_TITLE"),
								MessageBoxButtons.OK, MessageBoxIcon.Stop); 
							return false;
						} 
					}
					// open archive file selected by user.
					archive = new ArchivedNotes();
					archive.ReadXml(archiveFile);
					return true;
				} 
				//archive was already loaded
				return true;
			}
			catch (IOException ioe)
			{
				//display message here about no permissions or other file access problem
				string text = string.Format(MainResource.GetString("ERROR_WRONG_FILE"), Properties.Settings.Default.ArchiveFile, ioe.Message);
				MessageBox.Show(text, MainResource.GetString("APP_TITLE"),
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			} 
		}

		/// <summary>
		/// ArchiveForm is disposed set ArchiveForm class holder and Archive dataset to null.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		internal void archiveForm_Disposed(object sender, EventArgs e)
		{
			archiveForm = null;
            archive = null;
		}

		public void SetNotesBackgroundColor()
		{
			if (0 != this.Count)
			{
                Color backColor = Properties.Settings.Default.BackgroundColor;

				IEnumerator myEnum = this.GetEnumerator();
				while (myEnum.MoveNext())
				{
					Note note = (Note) myEnum.Current;
					if (note.BackgroundColor == oldBackColor)
					{
						note.BackgroundColor = backColor;
						note.MyForm.BackgroundColor = backColor;
					}
				}
			}
		}

		public void SetNotesTextColor()
		{
			if (0 != this.Count)
			{
                Color textColor = Properties.Settings.Default.TextColor;
				IEnumerator myEnum = this.GetEnumerator();
				while (myEnum.MoveNext())
				{
					Note note = (Note) myEnum.Current;
					if (note.TextColor == oldTextColor)
					{
						note.TextColor = textColor;
						note.MyForm.TextColor = textColor;
					}
				}
			}
		}
		/// <summary>
		/// Method sets desknotes opacity according to value set by user in a settings dialog.
		/// </summary>
		internal void SetOpacity()
		{
			if (0 != this.Count)
			{
				IEnumerator myEnum = this.GetEnumerator();
				while (myEnum.MoveNext())
				{
					Note note = (Note) myEnum.Current;
					note.MyForm.Opacity = Properties.Settings.Default.Opacity;
                    note.MyForm.RemoveResizableBorder();
				}
			}
		}

        private void SetNotesFont()
        {
            if (0 != this.Count)
            {
                IEnumerator myEnum = this.GetEnumerator();
                while (myEnum.MoveNext())
                {
                    Note note = (Note)myEnum.Current;
                    if (null == note.ExtraFont)
                    {
                        note.MyForm.txtNote.Font = Properties.Settings.Default.Font;
                    }
                }
            }
        }

		/// <summary>
		/// Method updates one category with new name
		/// </summary>
		/// <param name="id">ID of category to be updated</param>
		/// <param name="text">New text to be a category name</param>
		/// <param name="isExpanded">Indicates expanded state if true, otherwise collapsed</param>
		/// <returns></returns>
		internal bool UpdateCategory(long id, TreeNode parent, string text, bool isExpanded)
		{
			ArchivedNotes.CategoryDataTable categories = Archive.Category;
			ArchivedNotes.CategoryRow[] catRow = (ArchivedNotes.CategoryRow[]) categories.Select("id = " + id.ToString());
			if (catRow.Length != 0)
			{
				catRow[0].Name = text;
				if (parent != null)
					catRow[0].parentID = ((DeskNotesTreeNode)parent).ID;
				catRow[0].IsExpanded = isExpanded;
				catRow[0].AcceptChanges();
				return true;
			}
			return false;
		}

		/// <summary>
		/// Method updates just IsExpanded state of category node when it is changed
		/// in Archive tree view
		/// </summary>
		/// <param name="id">ID of category to be updated</param>
		/// <param name="isExpanded">Indicates expanded state if true, otherwise collapsed</param>
		/// <returns></returns>
		internal bool UpdateCategoryExpanded(long id, bool isExpanded)
		{
			ArchivedNotes.CategoryDataTable categories = Archive.Category;
			ArchivedNotes.CategoryRow[] catRow = (ArchivedNotes.CategoryRow[]) categories.Select("id = " + id.ToString());
			if (catRow.Length != 0)
			{
				catRow[0].IsExpanded = isExpanded;
				catRow[0].AcceptChanges();
				return true;
			}
			return false;
		}

		/// <summary>
		/// Updates ArchivedNote in dataset
		/// </summary>
		/// <param name="id"></param>
		/// <param name="catID"></param>
		/// <param name="text"></param>
		/// <returns></returns>
		internal bool UpdateArchivedNote(ArchivedNoteNode note, long catID) //string text, DateTime lastModificationTime)
		{
			ArchivedNotes.ArchivedNoteDataTable archNote = Archive.ArchivedNote;
			ArchivedNotes.ArchivedNoteRow[] noteRow = (ArchivedNotes.ArchivedNoteRow[]) archNote.Select("id = " + note.ID.ToString());
			if (noteRow.Length != 0)
			{
				noteRow[0].text = note.NoteText;
				noteRow[0].CategoryID = catID;
                noteRow[0].LastModificationTime = note.LastModificationTime;
                noteRow[0].backColor = note.PanelBackColor.ToArgb();
				noteRow[0].AcceptChanges();
				return true;
			}
			return false;
		}

		/// <summary>
		/// Removes archived note by its index including its position row.
		/// </summary>
		/// <param name="id">ID of archived note to be removed</param>
		/// <returns>true if removal was successfull otherwise false</returns>
		internal bool RemoveArchivedNote(long id)
		{
			ArchivedNotes.ArchivedNoteDataTable archNote = Archive.ArchivedNote;
			ArchivedNotes.ArchivedNoteRow[] noteRow = (ArchivedNotes.ArchivedNoteRow[]) archNote.Select("id = " + id.ToString());
			if (noteRow.Length != 0)
			{
				// Remove desktop positions if exist
				ArchivedNotes.PositionRow[] positions = noteRow[0].GetPositionRows();
				foreach (ArchivedNotes.PositionRow posRow in positions)
				{
					Archive.Position.RemovePositionRow(posRow);
					Archive.Position.AcceptChanges();
				}
				// Remove font records if exist
				ArchivedNotes.FontRow[] fonts = noteRow[0].GetFontRows();
				foreach (ArchivedNotes.FontRow fontRow in fonts)
				{
					Archive.Font.RemoveFontRow(fontRow);
					Archive.Font.AcceptChanges();
				}
				//Remove archived note itself
				Archive.ArchivedNote.RemoveArchivedNoteRow(noteRow[0]);
				Archive.ArchivedNote.AcceptChanges();
				return true;
			}
			return false;
		}

		/// <summary>
		/// Method creates tree from archive.
		/// If there is no category in the dataset then it creates the default one.
		/// </summary>
		internal CategoryNode CreateTreeFromArchive(ref TreeNode selectedNode)
		{
			CategoryNode rootCategory;
			// If there is no category inside Category table then we have to create
			// default one otherwise read all of them from table
			ArchivedNotes.CategoryDataTable categories = Archive.Category;
			if (categories.Count == 0)
			{
				// create default category node
				rootCategory = new CategoryNode(MainResource.GetString("DEFAULT_ARCHIVE_CATEGORY"));
				// new category is written into data set immediately.
				ArchivedNotes.CategoryRow row = Archive.Category.AddCategoryRow(0, rootCategory.Text, false);
				// reset parentID to null for very first default category
				row.parentID = -1;
				categories.AcceptChanges();
				SaveArchive();
			}
			else 
				// read top level category from archive dataset
			{
				ArchivedNotes.CategoryRow[] parentCat = (ArchivedNotes.CategoryRow[]) categories.Select("parentID = -1");
				rootCategory = new CategoryNode( parentCat[0].Name, parentCat[0].ID );
				if (!parentCat[0].IsIsExpandedNull())
				{
					rootCategory.IsExpanded = parentCat[0].IsExpanded;
				}
			}

			AddCategories(rootCategory, rootCategory.ID, ref selectedNode);
			return rootCategory;						
		}

		/// <summary>
		/// Method adds recursively all categories and archived nodes
		/// into given treenode.
		/// </summary>
		/// <param name="parent">Tree node which is a parent for all subcategories and Notes that could be found under it.</param>
		/// <param name="id">Category ID used to select it child categories and Notes</param>
		/// <param name="selectedNode">Node which has to be selected after last open of archive</param>
		internal void AddCategories(CategoryNode parent, long id, ref TreeNode selectedNode)
		{
			CategoryNode catNode;
			// Add sub categories of this category
			ArchivedNotes.CategoryDataTable tblCategory = Archive.Category;
			
			ArchivedNotes.CategoryRow[] categories = (ArchivedNotes.CategoryRow[]) tblCategory.Select( "parentID = " + id.ToString() );
			if (categories.Length >= 1 )
			{
				foreach (ArchivedNotes.CategoryRow catRow in categories)
				{
                    try
                    {
                        catNode = new CategoryNode(catRow.Name, catRow.ID);
                    } // Fallback when category name is not defined for some reason, but Category exists.
                    catch (System.Data.StrongTypingException ex)
                    {
                        catNode = new CategoryNode(Resources.AppResource.DEFAULT_ARCHIVE_CATEGORY, catRow.ID);
                    }

					if (!catRow.IsIsExpandedNull())
					{
						catNode.IsExpanded = catRow.IsExpanded;
					}
					if ((Properties.Settings.Default.SelectedNodeType == DeskNotesTreeNode.CATEGORY) && (Properties.Settings.Default.SelectedNodeId == catRow.ID))
						selectedNode = (TreeNode) catNode;

					parent.Nodes.Add(catNode);
					AddCategories( catNode, catRow.ID, ref selectedNode );
				}
			}
					
			// Add also Archived DeskNotes here
			ArchivedNotes.ArchivedNoteDataTable tblNotes = Archive.ArchivedNote;
			ArchivedNotes.ArchivedNoteRow[] noteRows = (ArchivedNotes.ArchivedNoteRow[]) tblNotes.Select( "CategoryID = " +  id.ToString());
			if (noteRows.Length >= 1 )
			{
				ArchivedNoteNode archNode;
				foreach(ArchivedNotes.ArchivedNoteRow noteRow in noteRows)
				{
					archNode = new ArchivedNoteNode(noteRow.text,noteRow.ID);
					archNode.ArchivedTime = noteRow.dateArchived;
                    if (!noteRow.IsLastModificationTimeNull())
                        archNode.LastModificationTime = noteRow.LastModificationTime;
					if (!noteRow.IsbackColorNull())
						archNode.PanelBackColor = Color.FromArgb(noteRow.backColor);
					if (!noteRow.IstextColorNull())
						archNode.PanelTextColor = Color.FromArgb(noteRow.textColor);
					archNode.Font = GetFont(noteRow);

					if ((Properties.Settings.Default.SelectedNodeType == DeskNotesTreeNode.NOTE) && (Properties.Settings.Default.SelectedNodeId == noteRow.ID))
						selectedNode = (TreeNode) archNode;
					parent.Nodes.Add( archNode );
				}
			}
		
		}

		/// <summary>
		/// Read font information from archive
		/// </summary>
		/// <param name="noteRow">ArchviedNote row to be checked for font info.</param>
		/// <returns>Font read or null if special font does not exist.</returns>
		private Font GetFont(ArchivedNotes.ArchivedNoteRow noteRow)
		{
			ArchivedNotes.FontRow[] fonts = noteRow.GetFontRows();
			if (fonts.Length > 0)
			{
				FontStyle fontStyle = FontStyle.Regular;
				bool bold = fonts[0].bold;
				bool italic = fonts[0].italic;
				bool strikeout = fonts[0].strikeout;
				bool underline = fonts[0].underline;
				fontStyle |= bold ? FontStyle.Bold : 0;
				fontStyle |= italic ? FontStyle.Italic : 0;
				fontStyle |= strikeout ? FontStyle.Strikeout : 0;
				fontStyle |= underline ? FontStyle.Underline : 0;
				Font font = new Font(fonts[0].name, fonts[0].size, fontStyle);
				return font;
			}
			return null;
		}

		/// <summary>
		/// Saves one desknote into archive.
		/// </summary>
		/// <param name="node">ArchivedNoteNode created in tree</param>
		/// <param name="note">DeskNote from desktop being archived</param>
		/// <returns></returns>
		internal bool ArchiveDeskNote(ArchivedNoteNode node, Note note)
		{
			ArchivedNotes.ArchivedNoteRow archNote = archive.ArchivedNote.AddArchivedNoteRow(((DeskNotesTreeNode)node.Parent).ID,note.Text,
				note.TextColor.ToArgb(), note.BackgroundColor.ToArgb(),node.ArchivedTime, node.LastModificationTime);
            ArchivedNotes.PositionRow position = archive.Position.AddPositionRow(archNote,note.Position.X, note.Position.Y, 
				note.Position.Width, note.Position.Height);
			if (null != note.ExtraFont)
			{
				ArchivedNotes.FontRow font = archive.Font.AddFontRow(note.ExtraFont.Name, note.ExtraFont.Size,
					note.ExtraFont.Bold, note.ExtraFont.Italic, note.ExtraFont.Strikeout, 
					note.ExtraFont.Underline, archNote);
			}
			archive.AcceptChanges();
			node.ID = archNote.ID;
			return true;
		}

		/// <summary>
		/// Adds newly created category node into archive.
		/// </summary>
		/// <param name="node">Tree view node created inside tree and being added into archive</param>
		/// <returns>true when operation completes</returns>
		internal bool AddArchiveCategory(CategoryNode node)
		{
			ArchivedNotes.CategoryRow catRow = archive.Category.AddCategoryRow(((CategoryNode)node.Parent).ID, node.Text, node.IsExpanded);
			archive.Category.AcceptChanges();
			node.ID = catRow.ID;
			return true;
		}

		/// <summary>
		/// Recusrsively removes category and all its subnodes from archive.
		/// </summary>
		/// <param name="id">Category node id to be removed</param>
		/// <returns>true when category was successfully removed.</returns>
		internal bool RemoveCategory(long id)
		{
			ArchivedNotes.CategoryRow[] categories = (ArchivedNotes.CategoryRow[]) archive.Category.Select("parentID = " + id.ToString());
			foreach(ArchivedNotes.CategoryRow catRow in categories)
			{
				this.RemoveCategory(catRow.ID);
			}
			ArchivedNotes.ArchivedNoteRow[] noteRows = (ArchivedNotes.ArchivedNoteRow[]) archive.ArchivedNote.Select("CategoryID = " + id.ToString());
			foreach(ArchivedNotes.ArchivedNoteRow note in noteRows)
			{
				ArchivedNotes.PositionRow[] position = note.GetPositionRows();
				archive.Position.RemovePositionRow(position[0]);

				ArchivedNotes.FontRow[] fonts = note.GetFontRows();
                if (fonts.Length > 0)
				    archive.Font.RemoveFontRow(fonts[0]);

				archive.ArchivedNote.RemoveArchivedNoteRow(note);
			}
			ArchivedNotes.CategoryRow[] category = (ArchivedNotes.CategoryRow[]) archive.Category.Select("ID = " + id.ToString());
			archive.Category.RemoveCategoryRow(category[0]);
			archive.AcceptChanges();
			return true;
		}

		/// <summary>
		/// Get all Note information from archive for its recreation 
		/// on desktop. Note is only copied to newly created one.
		/// </summary>
		/// <param name="id">id of Archived Note </param>
		/// <returns>Note created from archived data or null</returns>
		internal Note GetNoteFromArchive(long id)
		{
			ArchivedNotes.ArchivedNoteDataTable archNote = Archive.ArchivedNote;
			ArchivedNotes.ArchivedNoteRow[] noteRow = (ArchivedNotes.ArchivedNoteRow[]) archNote.Select("id = " + id.ToString());
			if (noteRow.Length != 0)
			{
				Color textColor = Color.FromArgb(noteRow[0].textColor);
				Color backColor = Color.FromArgb(noteRow[0].backColor);
				Rectangle position = Note.DefaultPosition;
				ArchivedNotes.PositionRow[] pos = noteRow[0].GetPositionRows();
				if (pos.Length != 0)
				{
					position = new Rectangle(pos[0].x, pos[0].y, pos[0].width, pos[0].height);
				}
				Font font = this.GetFont(noteRow[0]);
				Note note;
				if (null != font)
					note = new Note(0,noteRow[0].text, position, backColor, textColor, font);
				else
					note = new Note(0,noteRow[0].text, position, backColor, textColor);
                if (!noteRow[0].IsLastModificationTimeNull())
                {
                    note.LastModificationTime = noteRow[0].LastModificationTime;
                }
				return note;
			}
			return null;
		}

        /// <summary>
        /// Creates time stamp string that can be placed into desknote.
        /// </summary>
        internal string TimeStamp
        {
            get
            {
                System.Globalization.DateTimeFormatInfo myDTFI = System.Windows.Forms.Application.CurrentCulture.DateTimeFormat;
                System.Text.StringBuilder timeStamp = new System.Text.StringBuilder(20);
                timeStamp.Append(DateTime.Now.ToString(myDTFI.ShortDatePattern));
                if (false == Properties.Settings.Default.InsertOnlyDate)
                {
                    timeStamp.Append(" ");
                    timeStamp.Append(DateTime.Now.ToString(myDTFI.ShortTimePattern));
                }
                return timeStamp.ToString();
            }
        }

		/// <summary>
		/// Inserts timestamp into given text.
		/// </summary>
		/// <param name="text">text to be modified by timestamp</param>
		/// <param name="position">position at given text where timestamp has to be inserted</param>
		/// <returns>text with timestamp inserted</returns>
		internal string InsertTimestamp(string text, ref int position)
		{
			System.Globalization.DateTimeFormatInfo myDTFI = System.Windows.Forms.Application.CurrentCulture.DateTimeFormat;
			System.Text.StringBuilder timeStamp = new System.Text.StringBuilder(20);
			timeStamp.Append(DateTime.Now.ToString(myDTFI.ShortDatePattern));
			if (false == Properties.Settings.Default.InsertOnlyDate)
			{
				timeStamp.Append(" ");
				timeStamp.Append(DateTime.Now.ToString(myDTFI.ShortTimePattern));
			}
			int newPosition = position;
			if (0 == position)
			{   // inserts at the very beginning of text
				timeStamp.Append("\n\n");
				newPosition += timeStamp.Length - 1;
			} 
			else
			{
				if ("\n" == text.Substring(position-1, 1))
				{   // if end of line is before insertion point then don't 
					// insert \n at the beginning of time stamp but \n\n at the end
					timeStamp.Append("\n\n");
					newPosition += timeStamp.Length - 1;
				} 
				else
				{   // insert one \n at the beginning and one at the end.
					timeStamp.Insert(0, "\n");
					timeStamp.Append("\n");
					newPosition += timeStamp.Length;
				}
			}
			string modifiedText = text.Insert(position, timeStamp.ToString()); 
			position = newPosition;
			return modifiedText;
		}

		internal RemindersForm RemindersWindow
		{
			get 
			{
				if (null == remindersForm)
				{
					remindersForm = new RemindersForm(this);
					//remindersForm.Show();
				}
				return remindersForm;
			}
		}

		/// <summary>
		/// When reminders form is closed then it cannot hold reference to it.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public void RemindersDisposed(object sender, EventArgs e)
		{
			remindersForm = null;
		}
		
		/// <summary>
		/// Delegate handling actual printing of notes.
		/// </summary>
		delegate void PrintInBackgroundDelegate();
		
		/// <summary>
		/// Prints more notes from array.
		/// </summary>
		/// <param name="notes">notes array</param>
		/// <param name="disableMe">menu to be disabled during background printing</param>
		internal void PrintNotes(IPrintableNote[] notes, params System.Windows.Forms.ToolStripMenuItem[] disableMe)
		{
			notePrintDoc = new NotePrintDocument(this.pageSettings, notes);
			System.Windows.Forms.PrintDialog printDlg = new PrintDialog();
			printDlg.AllowSelection = true;
			printDlg.AllowSomePages = true;
			printDlg.Document = notePrintDoc;
			if (DialogResult.OK == printDlg.ShowDialog())
			{
				PrintController printController = new StandardPrintController();
				notePrintDoc.PrintController = printController;
				disabledPrintMenus = disableMe;
				PrintInBackgroundDelegate d = new PrintInBackgroundDelegate(PrintInBackground);
				d.BeginInvoke(new AsyncCallback(PrintInBackgroundComplete), null);
			}

		}

		/// <summary>
		/// Prints one note from desktop or from archive on background.
		/// </summary>
		/// <param name="note">Note to be printed</param>
		/// <param name="disableMe">Print menu that has to be disabled when doing backg. printing.</param>
		internal void PrintOneNote(IPrintableNote note, params System.Windows.Forms.ToolStripMenuItem[] disableMe)
		{
			
			notePrintDoc = new NotePrintDocument(this.pageSettings, note);
			System.Windows.Forms.PrintDialog printDlg = new PrintDialog();
			printDlg.AllowSelection = true;
			printDlg.AllowSomePages = true;
			printDlg.Document = notePrintDoc;
			if (DialogResult.OK == printDlg.ShowDialog())
			{
				PrintController printController = new StandardPrintController();
				notePrintDoc.PrintController = printController;
				disabledPrintMenus = disableMe;
				PrintInBackgroundDelegate d = new PrintInBackgroundDelegate(PrintInBackground);
				d.BeginInvoke(new AsyncCallback(PrintInBackgroundComplete), null);
			}
		}

		private void PrintInBackground()
		{
			try
			{
				EnablePrintMenus = false;
				notePrintDoc.Print();
			}
			catch (Exception e)
			{
				EnablePrintMenus = true;
				MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void PrintInBackgroundComplete(IAsyncResult r)
		{
			EnablePrintMenus = true;
			notePrintDoc.Dispose();
			notePrintDoc = null;
		}

		/// <summary>
		/// Disables Print menu item during background print. 
		/// When done it enables it.
		/// </summary>
		private bool EnablePrintMenus
		{ 
			set 
			{
                if (null != disabledPrintMenus)
                {
                    foreach (System.Windows.Forms.ToolStripMenuItem menu in disabledPrintMenus)
                        if (null != menu)
                            menu.Enabled = value;
                }
			}
		}

        
    }

    /// <summary>
    /// Converts plain text to rtf and rtf to plain text.
    /// </summary>
    public static class RtfConverter
    {
        internal static string RTF_TAG = "{\\rtf1\\";
        /// <summary>
        /// Converts rtf formatted text to plain text and return it.
        /// </summary>
        /// <param name="rtf">RTF formatted string</param>
        /// <returns>plain text</returns>
        public static string Text(string rtf)
        {
            RichTextBox box = new RichTextBox();
            if (rtf.StartsWith(RTF_TAG))
                box.Rtf = rtf;
            else
                box.Text = rtf;
            return box.Text;
        }
        /// <summary>
        /// Converts plain text into RTF and returns RTF formatted text.
        /// </summary>
        /// <param name="text">Plain text</param>
        /// <returns>RTF formatted string</returns>
        public static string Rtf(string text)
        {
            RichTextBox box = new RichTextBox();
            if (text.StartsWith(RTF_TAG))
                box.Rtf = text;
            else
                box.Text = text;
            return box.Rtf;
        }

    }

	public class MainResource
	{
		public static ResourceManager mainResource = new ResourceManager("DeskNotes.Resources.AppResource",System.Reflection.Assembly.GetExecutingAssembly());

		public static string GetString(string key)
		{
			return mainResource.GetString(key);
		}

	}
}
